package com.jeeplus.finance.ccpm.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jeeplus.finance.ccpm.service.TranspondService;
import com.jeeplus.finance.ccpm.vo.TranspondDTO;
import com.jeeplus.finance.invoice.domain.CwFinanceInvoice;
import com.jeeplus.finance.invoice.domain.CwFinanceInvoiceReceivables;
import com.jeeplus.finance.invoice.mapper.CwFinanceInvoiceMapper;
import com.jeeplus.finance.invoice.mapper.CwFinanceInvoiceReceivablesMapper;
import com.jeeplus.finance.invoice.service.CwFinanceInvoiceService;
import com.jeeplus.finance.invoice.service.dto.CwFinanceInvoiceDTO;
import com.jeeplus.finance.invoice.service.dto.CwFinanceInvoiceReceivablesDTO;
import com.jeeplus.finance.invoice.service.mapstruct.CwFinanceInvoiceReceivablesWrapper;
import com.jeeplus.sys.feign.IUserApi;
import com.jeeplus.sys.service.dto.UserDTO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping(value = "/transpond")
public class Transport {

    @Autowired
    private CwFinanceInvoiceService cwFinanceInvoiceService;

    @Autowired
    private TranspondService transpondService;

    @Resource
    private CwFinanceInvoiceMapper cwFinanceInvoiceMapper;

    @Resource
    private CwFinanceInvoiceReceivablesMapper cwFinanceInvoiceReceivablesMapper;


    @RequestMapping(value = "detailFinance", method = RequestMethod.GET)
    public Object detailDistribute(@RequestParam String id, @RequestParam String processDefKey) {
        return cwFinanceInvoiceService.detailDistribute(id, processDefKey);
    }

    //查询所有的发票信息
    @RequestMapping(value = "FinanceList", method = RequestMethod.POST)
    public Object FinanceList() {
        return cwFinanceInvoiceService.findListNew();
    }


    /**
     * 审核分发操作
     * @return
     */
    @RequestMapping(value = "auditDistribute", method = RequestMethod.POST)
    @ResponseBody
    public Map<String,Object> auditDistribute(@RequestBody TranspondDTO transpondDTO) {
        Map<String, Object> res = transpondService.auditDistribute(transpondDTO);
        return res;
    }

    /**
     * 修改收款状态
     */
    @Transactional(readOnly = false)
    @RequestMapping(value = "saveReceivable", method = RequestMethod.POST)
    public void saveReceivable(@RequestBody String json) throws Exception {
        CwFinanceInvoice workInvoice = new CwFinanceInvoice();
        if (null != json){
            Map<String, Object> invoice = JSONObject.parseObject(json);
            if (Objects.nonNull(invoice)){
                if (Objects.nonNull(invoice.get("id"))){
                    workInvoice.setId(invoice.get("id").toString());
                }
                if (Objects.nonNull(invoice.get("receiptMoneyDate"))){
                    String billingDate = invoice.get("receiptMoneyDate").toString();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    Date date = sdf.parse(billingDate);
                    workInvoice.setReceivablesDate(date);
                }
                if (Objects.nonNull(invoice.get("receiptMoney"))){
                    workInvoice.setReceivablesStatus(invoice.get("receiptMoney").toString());
                }
            }
            if (workInvoice != null){
                cwFinanceInvoiceMapper.update(workInvoice, new QueryWrapper<CwFinanceInvoice>().lambda().eq(CwFinanceInvoice::getId, workInvoice.getId()));
            }
        }


    }

    /**
     * 保存收款明细
     */
    @Transactional(readOnly = false)
    @RequestMapping(value = "saveReceipt", method = RequestMethod.POST)
    public void saveReceipt(@RequestBody String json) throws Exception {
        CwFinanceInvoiceDTO workInvoice = new CwFinanceInvoiceDTO();
        String invoiceId = "";
        if (null != json){

            Map<String, Object> resp = JSONObject.parseObject(json);
            if (Objects.nonNull(resp)){
                if (Objects.nonNull(resp.get("workReceiptList"))){
                    String data = JSON.toJSONString(resp.get("workReceiptList"));
                    List<Object> baseList = JSON.parseArray(data,Object.class);
                    ArrayList<CwFinanceInvoiceReceivablesDTO> workInvoiceReceipts = new ArrayList<>();
                    if (Objects.nonNull(baseList)) {
                        for (Object base : baseList) {
                            CwFinanceInvoiceReceivablesDTO receipt = new CwFinanceInvoiceReceivablesDTO();
                            Map<String, Object> baseMap = JSONObject.parseObject(JSON.toJSONString(base));
                            if (Objects.nonNull(baseMap.get("invoiceId"))){
                                receipt.setInvoiceId(baseMap.get("invoiceId").toString());
                                invoiceId = baseMap.get("invoiceId").toString();
                            }
                            if (Objects.nonNull(baseMap.get("companyName"))){
                                receipt.setRemittanceUnit(baseMap.get("companyName").toString());
                            }
                            if (Objects.nonNull(baseMap.get("money"))){
                                receipt.setRemittanceAmount(baseMap.get("money").toString());
                            }
                            if (Objects.nonNull(baseMap.get("receiptDate"))){
                                String billingDate = baseMap.get("receiptDate").toString();
                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                                Date date = sdf.parse(billingDate);
                                receipt.setRemittanceDate(date);
                            }
                            if (Objects.nonNull(baseMap.get("createDate"))){
                                String billingDate = baseMap.get("createDate").toString();
                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                                Date date = sdf.parse(billingDate);
                                receipt.setCreateTime(date);
                            }
                            if (Objects.nonNull(baseMap.get("updateDate"))){
                                String billingDate = baseMap.get("updateDate").toString();
                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                                Date date = sdf.parse(billingDate);
                                receipt.setUpdateTime(date);
                            }
                            if (Objects.nonNull(baseMap.get("createName"))){
                                UserDTO userDTO = SpringUtil.getBean(IUserApi.class).getByLoginNameNotTenantId(baseMap.get("createName").toString());
                                receipt.setCreateBy(userDTO);
                            }
                            if (Objects.nonNull(baseMap.get("updateName"))){
                                UserDTO userDTO = SpringUtil.getBean(IUserApi.class).getByLoginNameNotTenantId(baseMap.get("updateName").toString());
                                receipt.setUpdateBy(userDTO);
                            }
                            receipt.setId(baseMap.get("id").toString());

                            workInvoiceReceipts.add(receipt);
                        }
                    }
                    workInvoice.setFinanceInvoiceReceivablesDTOList(workInvoiceReceipts);
                }
            }

        }
        if (workInvoice != null && CollectionUtil.isNotEmpty(workInvoice.getFinanceInvoiceReceivablesDTOList())){
            cwFinanceInvoiceReceivablesMapper.deleteByInvoiceId(invoiceId);
            workInvoice.getFinanceInvoiceReceivablesDTOList().stream().forEach(item->{
                CwFinanceInvoiceReceivables cwFinanceInvoiceReceivables = CwFinanceInvoiceReceivablesWrapper.INSTANCE.toEntity(item);
                cwFinanceInvoiceReceivablesMapper.insert(cwFinanceInvoiceReceivables);

            });
        }


    }

    /**
     * 检测服务是否启动
     */
    @GetMapping("checkConnect")
    public ResponseEntity<Map<String, String>> checkConnect() {
        Map<String, String> response = new HashMap<>();
        response.put("message", "success");
        return ResponseEntity.ok(response);
    }







}
