package com.jeeplus.finance.config;


import com.jeeplus.common.redis.RedisUtils;
import com.jeeplus.finance.ccpm.controller.CcpmList;
import com.jeeplus.finance.utils.Global;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class ConnectionService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private CcpmList ccpmList;

    private ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private boolean isConnected = false;

    // 启动时调用，开始定期检查连接
    public void startCheckingConnection() {
        scheduler.scheduleAtFixedRate(this::checkConnection, 0,  1, TimeUnit.HOURS);
    }

    private void checkConnection() {
        try {
            String path = Global.getConfig("CCPM_PATH");
            String url = path+"/a/ccpmList/ccpmList/checkConnect"; // MVC 项目的 URL
            ResponseEntity<Map<String, String>> forObject = restTemplate.exchange(url,HttpMethod.GET,null,new ParameterizedTypeReference<Map<String, String>>() {});

            Map<String, String> responseBody = forObject.getBody();
            String message = responseBody.get("message");
            if ("success".equals(message)) {
//                if (!isConnected) {
//                    RedisUtils.getInstance().set("connect", "true", 24); // 存储连接状态到 Redis
//                    System.out.println("Connected successfully.");
//                    try {
                        ccpmList.getCcpmMethod(); // 调用 initMethod()
//                    } catch (Exception e) {
//                        System.err.println("Error during initMethod execution: " + e.getMessage());
//                    }
//                }
//                isConnected = true; // 更新连接状态
            }
        } catch (Exception e) {
            isConnected = false; // 连接失败，更新状态
            RedisUtils.getInstance().set("connect", "false", 24); // 存储连接失败状态到 Redis
            log.error("Connection failed: " + e.getMessage());
        }
    }

    // 可选：停止检查连接的方法
    public void stopCheckingConnection() {
        if (scheduler != null) {
            scheduler.shutdown(); // 停止调度
        }
    }
}

